package largeModel

import (
	"encoding/json"
	"example.com/m/conf"
	"example.com/m/log"
	"example.com/m/models"
	"example.com/m/operate"
	"io"
	"net/http"
	"strings"
	"time"
)

type ModelHandler struct {
	dockerOp *operate.DockerOp
	client   *http.Client
}

func NewModelHandler(dockerOp *operate.DockerOp) *ModelHandler {
	return &ModelHandler{
		dockerOp: dockerOp,
		client:   &http.Client{},
	}
}

func (m *ModelHandler) MonitorModelInfo() {
	ticker := time.NewTicker(time.Second * 10)
	for {
		select {
		case <-ticker.C:
			modelResp, err := m.client.Get(conf.GetConfig().ApiUrl)
			if err != nil {
				log.Error("Error getting model info from client failed:", err)
				continue
			}
			bodyBytes, err := io.ReadAll(modelResp.Body)
			if err != nil {
				log.Error("Error reading model response failed:", err)
				continue
			}
			resp := &models.Resp{}
			err = json.Unmarshal(bodyBytes, resp)
			if err != nil {
				log.Error("Unmarshal model response failed:", err)
				continue
			}
			if resp.Code != http.StatusOK {
				log.Error("Response code :", resp.Code)
				continue
			}
			if resp.Data == nil || len(resp.Data) == 0 {
				log.Warn("Response data is empty")
				continue
			}
			modelInfosResp := resp.Data
			imageNameMap, err := m.dockerOp.PsImageNameMap()
			if err != nil {
				log.Error("Docker op ps images failed:", err)
				continue
			}
			reportTaskIds := make([]uint64, 0)
			maxLong := uint64(0)
			for _, modelInfo := range modelInfosResp {
				if modelInfo.ImageName == "" {
					continue
				}
				split := strings.Split(modelInfo.ImageName, ":")
				if len(split) != 2 {
					continue
				}
				if !imageNameMap[modelInfo.ImageName] {
					// todo: 判断机器资源是否够用
					isPull := isResourceEnough(modelInfo)
					// todo： 如果够用
					if isPull && modelInfo.PublishStatus == models.ModelPublishStatusYes {
						log.WithField("model image name", modelInfo.ImageName).Info("pulling image")
						go m.dockerOp.PullImage(modelInfo)
					}
				} else {
					log.WithField("name", modelInfo.ImageName).Info("The image name is already")
					reportTaskIds = append(reportTaskIds, modelInfo.TaskId)
					if modelInfo.TaskId > maxLong {
						maxLong = modelInfo.TaskId
					}
				}
				m.dockerOp.SignApi[modelInfo.ImageName] = modelInfo.SignUrl
			}
			m.dockerOp.ModelsInfo = modelInfosResp
			reportTaskIds = append(reportTaskIds, maxLong)
			m.dockerOp.ReportTaskIds = reportTaskIds
			ticker = time.NewTicker(time.Minute * 10)
		}
	}
}

func isResourceEnough(modelInfo *models.ModelInfo) bool {
	return true
}
